/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SAMPLINGPOINTSTRUCT_H
#define DPC_IMPL_TYPE_SAMPLINGPOINTSTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct SamplingPointStruct {
    ::Float x{ 0.0F };
    ::Float y{ 0.0F };
    ::Float heading{ 0.0F };
    ::Float slope{ 0.0F };
    ::Float crosslope{ 0.0F };
    ::Float curve{ 0.0F };
    ::Float offset{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(x);
        fun(y);
        fun(heading);
        fun(slope);
        fun(crosslope);
        fun(curve);
        fun(offset);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(x);
        fun(y);
        fun(heading);
        fun(slope);
        fun(crosslope);
        fun(curve);
        fun(offset);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("x", x);
        fun("y", y);
        fun("heading", heading);
        fun("slope", slope);
        fun("crosslope", crosslope);
        fun("curve", curve);
        fun("offset", offset);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("x", x);
        fun("y", y);
        fun("heading", heading);
        fun("slope", slope);
        fun("crosslope", crosslope);
        fun("curve", curve);
        fun("offset", offset);
    }

    friend bool operator==(const ::dpc::SamplingPointStruct& lhs, const ::dpc::SamplingPointStruct& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.x - rhs.x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.y - rhs.y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.heading - rhs.heading)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.slope - rhs.slope)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.crosslope - rhs.crosslope)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.curve - rhs.curve)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.offset - rhs.offset)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SAMPLINGPOINTSTRUCT_H
